@@ -1,13 +1,24 @@
 """Command-line support for Coverage."""
 
-import optparse, sys
+import optparse, re, sys, traceback
 
+from coverage.backward import sorted                # pylint: disable-msg=W0622
 from coverage.execfile import run_python_file
+from coverage.misc import CoverageException, ExceptionDuringRun
 
 
-class Opts:
+class Opts(object):
     """A namespace class for individual options we'll build parsers from."""
-    
+
+    append = optparse.Option(
+        '-a', '--append', action='store_false', dest="erase_first",
+        help="Append coverage data to .coverage, otherwise it is started "
+                "clean with each run."
+        )
+    branch = optparse.Option(
+        '', '--branch', action='store_true',
+        help="Measure branch coverage in addition to statement coverage."
+        )
     directory = optparse.Option(
         '-d', '--directory', action='store',
         metavar="DIR",
@@ -50,26 +61,31 @@ class Opts:
         )
     parallel_mode = optparse.Option(
         '-p', '--parallel-mode', action='store_true',
-        help="Include the machine name and process id in the .coverage "
-                "data file name."
+        help="Append the machine name, process id and random number to the "
+                ".coverage data file name to simplify collecting data from "
+                "many processes."
+        )
+    rcfile = optparse.Option(
+        '', '--rcfile', action='store',
+        help="Specify configuration file.  Defaults to '.coveragerc'"
         )
     timid = optparse.Option(
         '', '--timid', action='store_true',
         help="Use a simpler but slower trace method.  Try this if you get "
                 "seemingly impossible results!"
         )
-    append = optparse.Option(
-        '-a', '--append', action='store_false', dest="erase_first",
-        help="Append coverage data to .coverage, otherwise it is started "
-                "clean with each run."
+    version = optparse.Option(
+        '', '--version', action='store_true',
+        help="Display version information and exit."
         )
-    
+
+
 class CoverageOptionParser(optparse.OptionParser, object):
     """Base OptionParser for coverage.
-    
+
     Problems don't exit the program.
     Defaults are initialized for all options.
-    
+
     """
 
     def __init__(self, *args, **kwargs):
@@ -78,29 +94,36 @@ class CoverageOptionParser(optparse.OptionParser, object):
             )
         self.set_defaults(
             actions=[],
+            branch=None,
             directory=None,
             help=None,
             ignore_errors=None,
             omit=None,
             parallel_mode=None,
             pylib=None,
+            rcfile=True,
             show_missing=None,
             timid=None,
             erase_first=None,
+            version=None,
             )
 
         self.disable_interspersed_args()
-        self.help_fn = lambda: None
+        self.help_fn = self.help_noop
+
+    def help_noop(self, error=None, topic=None, parser=None):
+        """No-op help function."""
+        pass
 
     class OptionParserError(Exception):
         """Used to stop the optparse error handler ending the process."""
         pass
-    
+
     def parse_args(self, args=None, options=None):
         """Call optparse.parse_args, but return a triple:
-        
+
         (ok, options, args)
-        
+
         """
         try:
             options, args = \
@@ -108,7 +131,7 @@ class CoverageOptionParser(optparse.OptionParser, object):
         except self.OptionParserError:
             return False, None, None
         return True, options, args
-        
+
     def error(self, msg):
         """Override optparse.error so sys.exit doesn't get called."""
         self.help_fn(msg)
@@ -120,7 +143,7 @@ class ClassicOptionParser(CoverageOptionParser):
 
     def __init__(self):
         super(ClassicOptionParser, self).__init__()
-        
+
         self.add_action('-a', '--annotate', 'annotate')
         self.add_action('-b', '--html', 'html')
         self.add_action('-c', '--combine', 'combine')
@@ -137,6 +160,7 @@ class ClassicOptionParser(CoverageOptionParser):
             Opts.old_omit,
             Opts.parallel_mode,
             Opts.timid,
+            Opts.version,
         ])
 
     def add_action(self, dash, dashdash, action_code):
@@ -145,7 +169,7 @@ class ClassicOptionParser(CoverageOptionParser):
             callback=self._append_action
             )
         option.action_code = action_code
-        
+
     def _append_action(self, option, opt_unused, value_unused, parser):
         """Callback for an option that adds to the `actions` list."""
         parser.values.actions.append(option.action_code)
@@ -153,19 +177,19 @@ class ClassicOptionParser(CoverageOptionParser):
 
 class CmdOptionParser(CoverageOptionParser):
     """Parse one of the new-style commands for coverage.py."""
-    
+
     def __init__(self, action, options=None, defaults=None, usage=None,
                 cmd=None, description=None
                 ):
         """Create an OptionParser for a coverage command.
-        
+
         `action` is the slug to put into `options.actions`.
         `options` is a list of Option's for the command.
         `defaults` is a dict of default value for options.
         `usage` is the usage string to display in help.
         `cmd` is the command name, if different than `action`.
         `description` is the description of the command, for the help text.
-        
+
         """
         if usage:
             usage = "%prog " + usage
@@ -184,6 +208,10 @@ class CmdOptionParser(CoverageOptionParser):
         # results, and they will compare equal to objects.
         return (other == "<CmdOptionParser:%s>" % self.cmd)
 
+GLOBAL_ARGS = [
+    Opts.rcfile,
+    Opts.help,
+    ]
 
 CMDS = {
     'annotate': CmdOptionParser("annotate",
@@ -191,15 +219,34 @@ CMDS = {
             Opts.directory,
             Opts.ignore_errors,
             Opts.omit,
-            Opts.help,
-            ],
+            ] + GLOBAL_ARGS,
         usage = "[options] [modules]",
         description = "Make annotated copies of the given files, marking "
             "statements that are executed with > and statements that are "
             "missed with !."
         ),
 
-    'help': CmdOptionParser("help", [Opts.help],
+    'combine': CmdOptionParser("combine", GLOBAL_ARGS,
+        usage = " ",
+        description = "Combine data from multiple coverage files collected "
+            "with 'run -p'.  The combined results are written to a single "
+            "file representing the union of the data."
+        ),
+
+    'debug': CmdOptionParser("debug", GLOBAL_ARGS,
+        usage = "<topic>",
+        description = "Display information on the internals of coverage.py, "
+            "for diagnosing problems. "
+            "Topics are 'data' to show a summary of the collected data, "
+            "or 'sys' to show installation information."
+        ),
+
+    'erase': CmdOptionParser("erase", GLOBAL_ARGS,
+        usage = " ",
+        description = "Erase previously collected coverage data."
+        ),
+
+    'help': CmdOptionParser("help", GLOBAL_ARGS,
         usage = "[command]",
         description = "Describe how to use coverage.py"
         ),
@@ -209,66 +256,43 @@ CMDS = {
             Opts.directory,
             Opts.ignore_errors,
             Opts.omit,
-            Opts.help,
-            ],
+            ] + GLOBAL_ARGS,
         usage = "[options] [modules]",
         description = "Create an HTML report of the coverage of the files.  "
             "Each file gets its own page, with the source decorated to show "
             "executed, excluded, and missed lines."
         ),
-    
-    'combine': CmdOptionParser("combine", [Opts.help],
-        usage = " ",
-        description = "Combine data from multiple coverage files collected "
-            "with 'run -p'.  The combined results are stored into a single "
-            "file representing the union of the coverage."
-        ),
-
-    'debug': CmdOptionParser("debug", [Opts.help],
-        usage = "<topic>",
-        description = "Display information the internals of coverage.py, "
-            "for diagnosing problems. "
-            "Topics are 'data' to show a summary of the data collected in "
-            ".coverage, or 'sys' to show installation information."
-        ),
-
-    'erase': CmdOptionParser("erase", [Opts.help],
-        usage = " ",
-        description = "Erase previously collected coverage data."
-        ),
 
     'report': CmdOptionParser("report",
         [
             Opts.ignore_errors,
             Opts.omit,
             Opts.show_missing,
-            Opts.help,
-            ],
+            ] + GLOBAL_ARGS,
         usage = "[options] [modules]",
-        description = "Report coverage stats on modules."
+        description = "Report coverage statistics on modules."
         ),
 
     'run': CmdOptionParser("execute",
         [
             Opts.append,
+            Opts.branch,
             Opts.pylib,
             Opts.parallel_mode,
             Opts.timid,
-            Opts.help,
-            ],
-        defaults = {'erase_first':True},
+            ] + GLOBAL_ARGS,
+        defaults = {'erase_first': True},
         cmd = "run",
         usage = "[options] <pyfile> [program options]",
-        description = "Run a python program, measuring code execution."
+        description = "Run a Python program, measuring code execution."
         ),
-    
+
     'xml': CmdOptionParser("xml",
         [
             Opts.ignore_errors,
             Opts.omit,
             Opts.output_xml,
-            Opts.help,
-            ],
+            ] + GLOBAL_ARGS,
         cmd = "xml",
         defaults = {'outfile': 'coverage.xml'},
         usage = "[options] [modules]",
@@ -277,9 +301,12 @@ CMDS = {
     }
 
 
-class CoverageScript:
+OK, ERR = 0, 1
+
+
+class CoverageScript(object):
     """The command-line interface to Coverage."""
-    
+
     def __init__(self, _covpkg=None, _run_python_file=None, _help_fn=None):
         # _covpkg is for dependency injection, so we can test this code.
         if _covpkg:
@@ -287,13 +314,13 @@ class CoverageScript:
         else:
             import coverage
             self.covpkg = coverage
-        
+
         # _run_python_file is for dependency injection also.
         self.run_python_file = _run_python_file or run_python_file
-        
+
         # _help_fn is for dependency injection.
         self.help_fn = _help_fn or self.help
-        
+
         self.coverage = None
 
     def help(self, error=None, topic=None, parser=None):
@@ -306,7 +333,6 @@ class CoverageScript:
             print(parser.format_help().strip())
         else:
             # Parse out the topic we want from HELP_TOPICS
-            import re
             topic_list = re.split("(?m)^=+ (\w+) =+$", HELP_TOPICS)
             topics = dict(zip(topic_list[1::2], topic_list[2::2]))
             help_msg = topics.get(topic, '').strip()
@@ -317,15 +343,14 @@ class CoverageScript:
 
     def command_line(self, argv):
         """The bulk of the command line interface to Coverage.
-        
+
         `argv` is the argument list to process.
 
         Returns 0 if all is well, 1 if something went wrong.
 
         """
         # Collect the command-line options.
-        OK, ERR = 0, 1
-        
+
         if not argv:
             self.help_fn(topic='minimum_help')
             return OK
@@ -367,6 +392,11 @@ class CoverageScript:
                 self.help_fn(topic='help')
             return OK
 
+        # Handle version.
+        if options.version:
+            self.help_fn(topic='version')
+            return OK
+
         # Check for conflicts and problems in the options.
         for i in ['erase', 'execute']:
             for j in ['annotate', 'html', 'report', 'combine']:
@@ -391,16 +421,18 @@ class CoverageScript:
         if not args_allowed and args:
             self.help_fn("Unexpected arguments: %s" % " ".join(args))
             return ERR
-        
+
         if 'execute' in options.actions and not args:
             self.help_fn("Nothing to do.")
             return ERR
-            
+
         # Do something.
         self.coverage = self.covpkg.coverage(
-            data_suffix = bool(options.parallel_mode),
+            data_suffix = options.parallel_mode,
             cover_pylib = options.pylib,
             timid = options.timid,
+            branch = options.branch,
+            config_file = options.rcfile,
             )
 
         if 'debug' in options.actions:
@@ -420,9 +452,12 @@ class CoverageScript:
                 elif info == 'data':
                     print("-- data ---------------------------------------")
                     self.coverage.load()
+                    print("path: %s" % self.coverage.data.filename)
+                    print("has_arcs: %r" % self.coverage.data.has_arcs())
                     summary = self.coverage.data.summary(fullpath=True)
                     if summary:
                         filenames = sorted(summary.keys())
+                        print("\n%d files:" % len(filenames))
                         for f in filenames:
                             print("%s: %d lines" % (f, summary[f]))
                     else:
@@ -460,7 +495,7 @@ class CoverageScript:
         if options.omit:
             omit = options.omit.split(',')
         report_args['omit_prefixes'] = omit
-        
+
         if 'report' in options.actions:
             self.coverage.report(
                 show_missing=options.show_missing, **report_args)
@@ -472,8 +507,6 @@ class CoverageScript:
                 directory=options.directory, **report_args)
         if 'xml' in options.actions:
             outfile = options.outfile
-            if outfile == '-':
-                outfile = None
             self.coverage.xml_report(outfile=outfile, **report_args)
 
         return OK
@@ -482,7 +515,7 @@ class CoverageScript:
 HELP_TOPICS = r"""
 
 == classic ====================================================================
-Coverage version %(__version__)s
+Coverage.py version %(__version__)s
 Measure, collect, and report on code coverage in Python programs.
 
 Usage:
@@ -528,14 +561,14 @@ Coverage data is saved in the file .coverage by default.  Set the
 COVERAGE_FILE environment variable to save it somewhere else.
 
 == help =======================================================================
-Coverage version %(__version__)s
+Coverage.py, version %(__version__)s
 Measure, collect, and report on code coverage in Python programs.
 
 usage: coverage <command> [options] [args]
 
 Commands:
     annotate    Annotate source files with execution information.
-    combine     Combine a number of data files. 
+    combine     Combine a number of data files.
     erase       Erase previously collected coverage data.
     help        Get help on using coverage.py.
     html        Create an HTML report.
@@ -550,13 +583,31 @@ For more information, see %(__url__)s
 == minimum_help ===============================================================
 Code coverage for Python.  Use 'coverage help' for help.
 
+== version ====================================================================
+Coverage.py, version %(__version__)s.  %(__url__)s
+
 """
 
 
 def main():
     """The main entrypoint to Coverage.
-    
+
     This is installed as the script entrypoint.
-    
+
     """
-    return CoverageScript().command_line(sys.argv[1:])
+    try:
+        status = CoverageScript().command_line(sys.argv[1:])
+    except ExceptionDuringRun:
+        # An exception was caught while running the product code.  The
+        # sys.exc_info() return tuple is packed into an ExceptionDuringRun
+        # exception.  Note that the Python interpreter doesn't print SystemExit
+        # tracebacks, so it's important that we don't also.
+        _, err, _ = sys.exc_info()
+        if not isinstance(err.args[1], SystemExit):
+            traceback.print_exception(*err.args)
+        status = ERR
+    except CoverageException:
+        _, err, _ = sys.exc_info()
+        print(err)
+        status = ERR
+    return status
